//
//  XEAppDelegate.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 12/18/12.
//  Copyright (c) 2012 XING AG. All rights reserved.
//

#import <JRSwizzle/JRSwizzle.h>
#import <XingSDK/XingSDK.h>
#import "XEAppDelegate.h"
#import "XingCredentials.h"
#import "XEMenuViewController.h"

@implementation RKManagedObjectStore (XE)

// http://www.mlsite.net/blog/?p=518 - this really sucks
- (void)mergeHelper:(NSNotification*)saveNotification {
    NSManagedObjectContext *context = XingSDK.manager.managedObjectStore.mainQueueManagedObjectContext;
    [context performBlock:^{
        for (NSManagedObject *object in saveNotification.userInfo[@"updated"]) {
            // It's possible that this exception will be thrown:
            // CoreData could not fulfill a fault for ...
            @try {
                [[context objectWithID:object.objectID] willAccessValueForKey:nil];
            }
            @catch (NSException *exception) {
                RKLogError(@"Exception: %@", exception);
            }
        }
        [context mergeChangesFromContextDidSaveNotification:saveNotification];
    }];
}

@end

@implementation XEAppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    RKLogConfigureByName("RestKit/Network", RKLogLevelOff);
    RKLogConfigureByName("RestKit/CoreData", RKLogLevelOff);
    [self setupXingSDK];
    [self setupWindow];
    [self setupAppearance];
    [self setupCoreDataMergeHelper];
    [[AFNetworkActivityIndicatorManager sharedManager] setEnabled:YES];

    if (XingSDK.session.isReady == NO) {
        [self showAuthViewControllerAnimated:NO];
    }
    else {
        [self showViewDeckControllerAnimated:NO];
    }

    return YES;
}

- (void)applicationWillTerminate:(UIApplication *)application {
    [MagicalRecord cleanUp];
}

#pragma mark - Setup methods

- (void)setupAppearance {
    [[UINavigationBar appearance] setBarStyle:UIBarStyleBlack];
    [[UINavigationBar appearance] setTintColor:UIColorXingCyan];
	[[UINavigationBar appearance] setBackgroundImage:[UIImage imageNamed:@"navigationBarBackground"]
                                       forBarMetrics:UIBarMetricsDefault];
}

- (void)setupWindow {
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.window.backgroundColor = [UIColor grayColor];
    [self.window makeKeyAndVisible];
}

- (void)setupXingSDK {
    XingSDK.session.consumerKey = kXingConsumerKey;
    XingSDK.session.consumerSecret = kXingConsumerSecret;
    XingSDK.session.callback = kXingCallback;

    __weak __typeof(&*self)weakSelf = self;
    [XingSDK.api.globalSignal subscribeError:^(NSError *error) {
        XingErrorMessage *xingError = (XingErrorMessage *)error.userInfo[XingAPIErrorKey];
        if (error.code == 401) {
            [weakSelf showAuthViewControllerAnimated:YES];
            return;
        }
        if (error.code == 403 && [xingError.errorName isEqualToString:@"RATE_LIMIT_EXCEEDED"]) {
            [XingSDK.manager cancelAllObjectRequestOperationsWithMethod:RKRequestMethodAny matchingPathPattern:@"/"];
            return;
        }
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"API Error"
                                                            message:xingError.errorMessage
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
        [alertView show];
    }];
}

- (void)setupCoreDataMergeHelper {
    [RKManagedObjectStore jr_swizzleMethod:@selector(handlePersistentStoreManagedObjectContextDidSaveNotification:)
                                withMethod:@selector(mergeHelper:)
                                     error:nil];
}

#pragma mark - Public methods

- (void)logout {
    [XingSDK.session logout];
    [self showAuthViewControllerAnimated:YES];
}

#pragma mark - Show view controller methods

- (void)showRootViewController:(UIViewController *)viewController
                      animated:(BOOL)animated
              animationOptions:(UIViewAnimationOptions)animationOptions {
    if (animated == NO) {
        self.window.rootViewController = viewController;
        return;
    }

    __weak __typeof(&*self)weakSelf = self;
    [UIView transitionWithView:self.window
                      duration:0.5
                       options:animationOptions
                    animations:^(void) {
                        weakSelf.window.rootViewController = viewController;
                    }
                    completion:nil];
}

- (void)showAuthViewControllerAnimated:(BOOL)animated {
    __weak __typeof(&*self)weakSelf = self;
    self.authViewController = [[[UINavigationController alloc] init] initWithRootViewController:[XingSDK.session authViewController:^(BOOL authorized, NSError *error) {
        if (authorized) {
            // Fetch the current user, once done, show the main view.
            [[XingSDK.api.loadUser deliverOn:RACScheduler.mainThreadScheduler] subscribeCompleted:^{
                [weakSelf showViewDeckControllerAnimated:YES];
            }];
        }
    }]];

    [self showRootViewController:self.authViewController
                        animated:animated
                animationOptions:UIViewAnimationOptionTransitionFlipFromRight];
}

- (void)showViewDeckControllerAnimated:(BOOL)animated {
    self.authViewController = nil;
    self.viewDeckController = [[XEViewDeckController alloc] init];

    [self showRootViewController:self.viewDeckController
                        animated:animated
                animationOptions:UIViewAnimationOptionTransitionFlipFromRight];
}

@end
