//
//  XEContactsCell.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 2/10/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XEContactsCell.h"

@implementation XEContactsCell

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]){
        self.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    return self;
}

#pragma mark - Setter

- (void)setUser:(XingEntityUser *)user {
    __weak __typeof(&*self)weakSelf = self;
    _user = user;
    self.textLabel.text = user.displayName;
    self.detailTextLabel.text = user.professionalExperience.primaryCompany.title;
    NSString *placeholder = [user.gender isEqualToString:@"m"] ? @"placeholder_user_male" : @"placeholder_user_female";
    NSURL *photoURL = [NSURL URLWithString:user.photoURL.large];
    if (photoURL.host && photoURL.path) {
        photoURL = [[NSURL alloc] initWithScheme:@"http" host:photoURL.host path:photoURL.path];
    }
    [self.imageView setImageWithURLRequest:[NSURLRequest requestWithURL:photoURL]
                          placeholderImage:[UIImage imageNamed:placeholder]
                                   success:^(NSURLRequest *request, NSHTTPURLResponse *response, UIImage *image) {
                                       if (request && response && image) {
                                           [UIView transitionWithView:weakSelf.imageView
                                                             duration:0.3f
                                                              options:UIViewAnimationOptionTransitionCrossDissolve
                                                           animations:^{
                                                               weakSelf.imageView.image = image;
                                                               [weakSelf setNeedsLayout];
                                                           }
                                                           completion:NULL];
                                       }
                                   } failure:nil];
}

#pragma mark - View life cycle

- (void)layoutSubviews {
    [super layoutSubviews];
    self.imageView.frame = CGRectInset(self.imageView.frame, 6, 6);
    self.imageView.contentMode = UIViewContentModeScaleAspectFit;
    self.imageView.clipsToBounds = YES;
}

@end
