//
//  XEContactsViewController.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 12/18/12.
//  Copyright (c) 2012 XING AG. All rights reserved.
//

#import <AFNetworking/UIImageView+AFNetworking.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "XEContactsCell.h"
#import "XEContactsViewController.h"

#define kUserFields @"id,display_name,page_name,photo_urls.large,gender,first_name,last_name,professional_experience.primary_company.name,professional_experience.primary_company.title"

@interface XEContactsViewController () <NSFetchedResultsControllerDelegate>

@property (nonatomic, strong) NSFetchedResultsController *fetchedResultsController;

@end

@implementation XEContactsViewController

#pragma mark - View life cycle

- (void)viewDidLoad {
    [super viewDidLoad];
}

#pragma mark - Private methods

- (void)deleteAllInContext:(NSManagedObjectContext *)context {
    ((XingEntityUser *)[self.user MR_inContext:context]).contacts = nil;
}

- (Class)entityClass {
    return [XingEntityUser class];
}

- (NSFetchedResultsController *)fetchedResultsController {
    if (_fetchedResultsController == nil) {
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"contactsInverse contains %@", self.user];
        _fetchedResultsController = [[self entityClass] MR_fetchAllGroupedBy:nil
                                                               withPredicate:predicate
                                                                    sortedBy:@"lastName"
                                                                   ascending:YES
                                                                    delegate:self];
        _fetchedResultsController.fetchRequest.sortDescriptors = @[[[NSSortDescriptor alloc] initWithKey:@"lastName" ascending:YES selector:@selector(caseInsensitiveCompare:)]];
        [_fetchedResultsController performFetch:nil];
    }
    return _fetchedResultsController;
}

- (NSSet *)items {
    return self.user.contacts;
}

- (RACSignal *)loadAllItems {
    __weak __typeof(&*self)weakSelf = self;
    NSDictionary *params = @{
    XingAPIParamLimit: @100,
    XingAPIParamOrderBy: @"last_name",
    XingAPIParamUserFields: kUserFields,
    };
    RACSignal *loadAllContacts = [XingSDK.api loadAllContactsWithParameters:params];
    [[loadAllContacts deliverOn:RACScheduler.mainThreadScheduler] subscribeNext:^(XingAPIResponse *response) {
        XingObjectContacts *contacts = response.resultObject;
        [NSManagedObjectContext.MR_defaultContext refreshObject:weakSelf.user mergeChanges:YES];
        float progress = (float)weakSelf.user.contacts.count / contacts.total.floatValue;
        [SVProgressHUD showProgress:progress status:@"" maskType:SVProgressHUDMaskTypeBlack];
    } completed:^{
        [SVProgressHUD dismiss];
    }];
    return loadAllContacts;
}

- (RACSignal *)loadItems {
    if ([self.user.userID isEqualToString:XingSDK.session.xingUserID] && self.user.contacts.count == 0) {
        return [self loadAllItems];
    }

    __weak __typeof(&*self)weakSelf = self;
    NSDictionary *params = @{
    XingAPIParamLimit: @30,
    XingAPIParamOffset: @(self.items.count),
    XingAPIParamOrderBy: @"last_name",
    XingAPIParamUserFields: kUserFields,
    };
    RACSignal *signal = [XingSDK.api loadContactsForUser:self.user.userID parameters:params];
    [[signal deliverOn:RACScheduler.mainThreadScheduler] subscribeNext:^(XingAPIResponse *response) {
        XingObjectContacts *contacts = response.resultObject;
        weakSelf.total = contacts.total.intValue;
    }];
    return signal;
}

- (void)pullToRefreshStarted {
    if ([self.user.userID isEqualToString:XingSDK.session.xingUserID]) {
        [SVProgressHUD showProgress:0.0f status:@"" maskType:SVProgressHUDMaskTypeBlack];
    }
}

#pragma mark - Table view data source

- (void)configureCell:(XEContactsCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    cell.user = [self.fetchedResultsController objectAtIndexPath:indexPath];
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(XEContactsCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath {
    [self configureCell:cell atIndexPath:indexPath];
}

- (XEContactsCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    XEContactsCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[XEContactsCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
    }
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    XingEntityUser *user = [self.fetchedResultsController objectAtIndexPath:indexPath];
    XEContactsViewController *vc = [[XEContactsViewController alloc] initWithUser:user];
    vc.title = user.displayName;
    [self.navigationController pushViewController:vc animated:YES];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 88.0f;
}

@end
