//
//  XEConversationsViewController.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 12/18/12.
//  Copyright (c) 2012 XING AG. All rights reserved.
//

#import <TDBadgedCell/TDBadgedCell.h>
#import "XEConversationsViewController.h"
#import "XEMessagesViewController.h"

@interface XEConversationsViewController () <NSFetchedResultsControllerDelegate>

@property (nonatomic, strong) NSFetchedResultsController *fetchedResultsController;

@end

@implementation XEConversationsViewController

#pragma mark - View life cycle

- (void)viewDidLoad {
    [super viewDidLoad];
}

#pragma mark - Private methods

- (void)deleteAllInContext:(NSManagedObjectContext *)context {
    ((XingEntityUser *)[self.user MR_inContext:context]).conversations = nil;
}

- (Class)entityClass {
    return [XingEntityConversation class];
}

- (NSFetchedResultsController *)fetchedResultsController {
    if (_fetchedResultsController == nil) {
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"user = %@", self.user];
        _fetchedResultsController = [[self entityClass] MR_fetchAllGroupedBy:nil
                                                               withPredicate:predicate
                                                                    sortedBy:@"updatedAt"
                                                                   ascending:NO
                                                                    delegate:self];
        [_fetchedResultsController performFetch:nil];
    }
    return _fetchedResultsController;
}

- (NSSet *)items {
    return self.user.conversations;
}

- (RACSignal *)loadItems {
    __weak __typeof(&*self)weakSelf = self;
    NSDictionary *params = @{
    XingAPIParamLimit: @15,
    XingAPIParamOffset: @(self.items.count),
    };
    RACSignal *signal = [XingSDK.api loadConversations:params];
    [[signal deliverOn:RACScheduler.mainThreadScheduler] subscribeNext:^(XingAPIResponse *response) {
        XingObjectConversations *conversations = response.resultObject;
        weakSelf.total = conversations.total.intValue;
    }];
    return signal;
}

#pragma mark - Table view data source

- (void)configureCell:(TDBadgedCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    XingEntityConversation *conversation = [self.fetchedResultsController objectAtIndexPath:indexPath];
    cell.textLabel.text = conversation.subject;
    cell.detailTextLabel.text = [NSString stringWithFormat:@"%u messages", conversation.messageCountValue];
    cell.badgeString = nil;
    if (conversation.unreadMessageCountValue > 0) {
        cell.badgeString = conversation.unreadMessageCount.stringValue;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    TDBadgedCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[TDBadgedCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
        cell.badgeColor = UIColorXingCyan;
        cell.badgeColorHighlighted = UIColorXingYellow;
        cell.badge.radius = 9.0f;
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    XingEntityConversation *conversation = [self.fetchedResultsController objectAtIndexPath:indexPath];
    XEMessagesViewController *vc = [[XEMessagesViewController alloc] initWithConversation:conversation];
    vc.title = conversation.subject;
    [self.navigationController pushViewController:vc animated:YES];
}

@end
