//
//  XEEntityViewController.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 12/18/12.
//  Copyright (c) 2012 XING AG. All rights reserved.
//

#import "XEEntityViewController.h"

#define kEntityName @"entityName"
#define kEntityCount @"entityCount"

@interface XEEntityViewController ()

@property (nonatomic, strong) NSArray *items;

@end

@implementation XEEntityViewController

#pragma mark - View life cycle

- (void)viewDidLoad {
    [super viewDidLoad];

    [self fetchItems];

    UIBarButtonItem *reloadButton;
    reloadButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh
                                                                 target:self
                                                                 action:@selector(reloadButtonAction:)];

    self.navigationItem.rightBarButtonItem = reloadButton;
}

#pragma mark - Private methods

- (void)fetchItems {
    NSManagedObjectModel *model = XingSDK.manager.managedObjectStore.managedObjectModel;
    NSArray *entities = [[model entities] valueForKey:@"name"];
    NSMutableArray *items = [NSMutableArray arrayWithCapacity:entities.count];
    for (NSString *entity in entities) {
        NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:entity];
        int count = [[NSManagedObjectContext MR_defaultContext] countForFetchRequest:fetchRequest error:nil];
        [items addObject:@{
             kEntityName: entity,
            kEntityCount: [NSString stringWithFormat:@"Entities in database: %u", count],
         }];
    }
    self.items = items;
    [self.tableView reloadData];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.items.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
    }
    NSDictionary *row = self.items[indexPath.row];
    cell.textLabel.text = row[kEntityName];
    cell.detailTextLabel.text = row[kEntityCount];
    return cell;
}

#pragma mark - Button actions

- (void)reloadButtonAction:(id)sender {
    [self fetchItems];
}

@end
