//
//  XEMenuViewController.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 12/18/12.
//  Copyright (c) 2012 XING AG. All rights reserved.
//

#import <TDBadgedCell/TDBadgedCell.h>
#import "XEAppDelegate.h"
#import "XEContactsViewController.h"
#import "XEConversationsViewController.h"
#import "XEEntityViewController.h"
#import "XEMenuViewController.h"
#import "XENetworkFeedController.h"
#import "XEViewDeckController.h"

@interface XEMenuViewController ()

@property (nonatomic, strong) NSArray *navigation;
@property (nonatomic, strong) NSIndexPath *selectedItem;
@property (nonatomic, strong) NSString *conversationsBadge;

@end

@implementation XEMenuViewController

#define kTitle @"title"
#define kHandler @"handler"
#define kBadge @"badge"

- (id)init {
    if (self = [super init]) {
        self.tableView.dataSource = self;
        self.tableView.delegate = self;
        self.tableView.tableFooterView = [UIView new];
        self.tableView.backgroundColor = UIColorFromRGB(0x3e3b35);
        self.tableView.separatorColor = [UIColor darkGrayColor];
    }
    return self;
}

- (UINavigationController *)defaultController {
    return [self controllerAtIndexPath:[NSIndexPath indexPathForItem:0 inSection:0]];
}

#pragma mark - View life cycle

- (void)viewDidLoad {
    [super viewDidLoad];
    [XingSDK.api.globalSignal subscribeNext:^(XingAPIResponse *response) {
        if ([response.resultObject isKindOfClass:XingObjectConversations.class]) {
            self.conversationsBadge = ((XingObjectConversations *)response.resultObject).unreadCount.stringValue;
        }
    }];
    [RACAble(self.conversationsBadge) subscribeNext:^(id x){ [self.tableView reloadData]; }];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    if (self.selectedItem == nil) {
        self.selectedItem = [NSIndexPath indexPathForRow:0 inSection:0];
    }
    [self.tableView selectRowAtIndexPath:self.selectedItem
                                animated:NO
                          scrollPosition:UITableViewScrollPositionNone];
}

#pragma mark - Table view data source

- (NSArray *)navigation {
    if (_navigation == nil) {
        _navigation = @[
        @{kTitle: @"Contacts", kHandler:^{
            return [XEContactsViewController class];
        }, kBadge:^{
            return nil;
        }},
        @{kTitle: @"Conversations", kHandler:^{
            return [XEConversationsViewController class];
        }, kBadge:^{
            return self.conversationsBadge;
        }},
        @{kTitle: @"Network Feed", kHandler:^{
            return [XENetworkFeedController class];
        }, kBadge:^{
            return nil;
        }},
        @{kTitle: @"Entities", kHandler:^{
            return [XEEntityViewController class];
        }, kBadge:^{
            return nil;
        }},
        @{kTitle: @"Logout", kHandler:^{
            XEAppDelegate *appDelegate = (XEAppDelegate *)[[UIApplication sharedApplication] delegate];
            [appDelegate logout];
            return nil;
        }, kBadge:^{
            return nil;
        }},
        ];
    }
    return _navigation;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.navigation.count;
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath {
    cell.backgroundColor = UIColorFromRGB(0x3e3b35);
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    TDBadgedCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[TDBadgedCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
        UIView *backgroundView = [[UIView alloc] initWithFrame:cell.frame];
        backgroundView.backgroundColor = UIColorXingYellow;
        cell.selectedBackgroundView = backgroundView;
        cell.textLabel.font = UIFontXing15bold;
        cell.textLabel.highlightedTextColor = [UIColor blackColor];
        cell.textLabel.textColor = [UIColor whiteColor];
        cell.badgeColorHighlighted = UIColorXingCyan;
        cell.badgeColor = [UIColor whiteColor];
        cell.badge.radius = 9.0f;
    }
    cell.textLabel.text = self.navigation[indexPath.row][kTitle];
    NSString *(^badge)() = self.navigation[indexPath.row][kBadge];
    cell.badgeString = badge();
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    self.selectedItem = indexPath;
    UINavigationController *newController = [self controllerAtIndexPath:indexPath];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        self.viewDeckController.centerController = newController;
    }
    else {
        [self.viewDeckController closeLeftViewBouncing:^(IIViewDeckController *viewDeckController) {
            viewDeckController.centerController = newController;
        }];
    }
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 44.0f;
}

#pragma mark - Private methods

- (UINavigationController *)controllerAtIndexPath:(NSIndexPath *)indexPath {
    UINavigationController *controller = nil;

    Class (^handler)() = self.navigation[indexPath.row][kHandler];
    Class viewControllerClass = handler();
    if (viewControllerClass == nil) {
        return controller;
    }

    UIViewController *currentController = [self currentRootController];
    if ([currentController isKindOfClass:viewControllerClass] == YES) {
        return (UINavigationController *)self.viewDeckController.centerController;
    }

    controller = [[viewControllerClass alloc] init];
    controller.title = self.navigation[indexPath.row][kTitle];

    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone) {
        UIBarButtonItem *menuButton;
        menuButton = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"menuButton"]
                                                      style:UIBarButtonItemStyleBordered
                                                     target:self
                                                     action:@selector(menuButtonAction:)];
        controller.navigationItem.leftBarButtonItem = menuButton;
    }

    return [[UINavigationController alloc] initWithRootViewController:controller];
}

- (UIViewController *)currentRootController {
    UINavigationController *navigationController = (UINavigationController *)self.viewDeckController.centerController;
    return navigationController.childViewControllers[0];
}

- (void)menuButtonAction:(id)sender {
    [self.navigationController.viewDeckController toggleLeftViewAnimated:YES];
}

- (XEViewDeckController *)viewDeckController {
    return (XEViewDeckController *)self.navigationController.viewDeckController;
}

@end