//
//  XEMessagesViewController.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 12/19/12.
//  Copyright (c) 2012 XING AG. All rights reserved.
//

#import "XEMessagesViewController.h"

@interface XEMessagesViewController () <NSFetchedResultsControllerDelegate>

@property (nonatomic, strong) NSFetchedResultsController *fetchedResultsController;

@end

@implementation XEMessagesViewController

- (id)initWithConversation:(XingEntityConversation *)inConversation {
    if (self = [super init]) {
        _conversation = inConversation;
    }
    return self;
}

#pragma mark - View life cycle

- (void)viewDidLoad {
    [super viewDidLoad];
}

#pragma mark - Private methods

- (void)deleteAllInContext:(NSManagedObjectContext *)context {
    ((XingEntityConversation *)[self.conversation MR_inContext:context]).messages = nil;
}

- (Class)entityClass {
    return [XingEntityMessage class];
}

- (NSFetchedResultsController *)fetchedResultsController {
    if (_fetchedResultsController == nil) {
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"conversation = %@", self.conversation];
        _fetchedResultsController = [[self entityClass] MR_fetchAllGroupedBy:nil
                                                               withPredicate:predicate
                                                                    sortedBy:@"messageID"
                                                                   ascending:YES
                                                                    delegate:self];
        [_fetchedResultsController performFetch:nil];
    }
    return _fetchedResultsController;
}

- (NSSet *)items {
    return self.conversation.messages;
}

- (RACSignal *)loadItems {
    __weak __typeof(&*self)weakSelf = self;
    NSDictionary *params = @{
    XingAPIParamLimit: @15,
    XingAPIParamOffset: @(self.items.count),
    };
    RACSignal *signal = [XingSDK.api loadMessages:self.conversation.conversationID parameters:params];
    [[signal deliverOn:RACScheduler.mainThreadScheduler] subscribeNext:^(XingAPIResponse *response) {
        XingObjectConversations *conversations = response.resultObject;
        weakSelf.total = conversations.total.intValue;
    }];
    return signal;
}

#pragma mark - Table view data source

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    XingEntityMessage *message = [self.fetchedResultsController objectAtIndexPath:indexPath];
    cell.textLabel.text = message.content;
    cell.detailTextLabel.text = [message.createdAt description];
}

@end
