//
//  XENetworkFeedController.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 1/13/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XENetworkFeedController.h"

@interface XENetworkFeedController () <NSFetchedResultsControllerDelegate>

@property (nonatomic, strong) NSFetchedResultsController *fetchedResultsController;

@end

@implementation XENetworkFeedController

#pragma mark - View life cycle

- (void)viewDidLoad {
    [super viewDidLoad];
    self.tableView.showsInfiniteScrolling = NO;
}

#pragma mark - Private methods

- (void)deleteAllInContext:(NSManagedObjectContext *)context {
    ((XingEntityUser *)[self.user MR_inContext:context]).networkActivities = nil;
}

- (Class)entityClass {
    return [XingEntityActivity class];
}

- (NSFetchedResultsController *)fetchedResultsController {
    if (_fetchedResultsController == nil) {
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"networkActivitiesInverse.userID contains %@", self.user.userID];
        _fetchedResultsController = [[self entityClass] MR_fetchAllGroupedBy:nil
                                                               withPredicate:predicate
                                                                    sortedBy:@"createdAt"
                                                                   ascending:NO
                                                                    delegate:self];
        [_fetchedResultsController performFetch:nil];
    }
    return _fetchedResultsController;
}

- (NSSet *)items {
    return self.user.networkActivities;
}

- (RACSignal *)loadItems {
    RACSignal *signal = [XingSDK.api loadNetworkFeed:@{XingAPIParamUserFields: XingAPIUserFieldsIDCard}];
    return signal;
}

#pragma mark - Table view data source

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    XingEntityActivity *activity = [self.fetchedResultsController objectAtIndexPath:indexPath];
    NSDictionary *actor = [activity.actors.lastObject objectData];
    cell.textLabel.text = actor[@"display_name"];
    cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ -> %@", activity.verb, [activity.objects.lastObject objectData][@"display_name"]];
    [cell.imageView setImageWithURL:[NSURL URLWithString:[actor valueForKeyPath:@"photo_urls.large"]]
                   placeholderImage:[UIImage imageNamed:@"placeholder_user_male"]];
}

@end
