//
//  XEViewDeckController.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 12/18/12.
//  Copyright (c) 2012 XING AG. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import "XEViewDeckController.h"
#import "XEMenuViewController.h"
#import "UIView+Helper.h"

@interface XEViewDeckController ()

@property (nonatomic, strong) XEMenuViewController *menuController;

@end

@implementation XEViewDeckController

#define kWidth 200.0f;

- (id)init {
    UINavigationController *centerController;
    UINavigationController *leftController;
    self.menuController = [[XEMenuViewController alloc] init];
    centerController = [self.menuController defaultController];
    leftController = [[UINavigationController alloc] initWithRootViewController:self.menuController];

    [leftController.navigationBar setBackgroundImage:[UIImage imageNamed:@"navigationBarMenuBackground"]
                                       forBarMetrics:UIBarMetricsDefault];

    self = [super initWithCenterViewController:centerController leftViewController:leftController];
    if (self == nil) {
        return nil;
    }

    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        self.resizesCenterView = YES;
        self.panningMode = IIViewDeckNoPanning;
    }
    else {
        self.centerhiddenInteractivity = IIViewDeckCenterHiddenNotUserInteractiveWithTapToClose;
    }

    self.sizeMode = IIViewDeckLedgeSizeMode;
    self.delegate = self;

    return self;
}

#pragma mark - View life cycle

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    self.maxSize = self.view.bounds.size.width - kWidth;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        [self openLeftViewAnimated:animated];
    }
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation
                                duration:(NSTimeInterval)duration {
    [self closeLeftViewAnimated:YES];
}

#pragma mark - IIViewDeckControllerDelegate

- (void)viewDeckController:(IIViewDeckController *)viewDeckController
               applyShadow:(CALayer *)shadowLayer
                withBounds:(CGRect)rect {
    shadowLayer.masksToBounds = NO;
    shadowLayer.shadowOpacity = 1.0;
    shadowLayer.shadowColor = [[UIColor blackColor] CGColor];
    shadowLayer.shadowOffset = CGSizeZero;
    shadowLayer.shadowPath = [[UIBezierPath bezierPathWithRect:rect] CGPath];
    shadowLayer.shadowRadius = 2;
}

- (void)viewDeckController:(IIViewDeckController*)viewDeckController
          willOpenViewSide:(IIViewDeckSide)viewDeckSide
                  animated:(BOOL)animated {
    if (viewDeckSide == IIViewDeckLeftSide) {
        self.menuController.view.width = kWidth;
    }
}

- (void)viewDeckController:(IIViewDeckController *)viewDeckController
         willCloseViewSide:(IIViewDeckSide)viewDeckSide
                  animated:(BOOL)animated {
    if (viewDeckSide == IIViewDeckLeftSide) {
        self.menuController.view.width = 320.0f;
    }
}

@end
